/*
 * Mach Operating System
 * Copyright (c) 1992, 1991 Carnegie Mellon University
 * Copyright (c) 1991 OMRON Corporation
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON AND OMRON ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON AND OMRON DISCLAIM ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie the
 * rights to redistribute these changes.
 */

/* console driver *very* losely based on luna88k.. */

#include <mach_kdb.h>

#define uart_tty cons_tty
#include <sys/types.h>
#include <kern/time_out.h>
#include <device/conf.h>
#include <device/tty.h>
#include <device/io_req.h>
#include <device/buf.h>		/* for struct uio (!) */

#include <machine/custom.h>
#include <machine/cons.h>
#include <machine/uart.h>

struct  tty     cons_tty;

/* a bit silly with one console... */
struct tty *curtty  = &uart_tty;       /* current tty struct */

extern void ttrstrt();

int 
cnopen (dev, flag, ior)
	unsigned dev;
	int	flag;
	io_req_t ior;
{
    return utopen (dev,flag,ior);
}


int 
cnclose (dev)
	unsigned dev;
{
    int	s = spltty();
    struct tty *tp;
    int ret;

    tp = &uart_tty;
    simple_lock(&tp->t_lock);
    ret = utclose(dev);
    ttyclose(&uart_tty);
    simple_unlock(&tp->t_lock);
    splx(s);
    return ret;
}


int 
cnwrite (dev, ior)
	unsigned dev;
	io_req_t ior;
{
  return utwrite(dev, ior);
}


int 
cnread (dev, ior)
	unsigned dev;
	io_req_t ior;
{
  struct tty *tp;

  tp = &uart_tty;
  tp->t_state |= TS_CARR_ON;
  return utread (dev, ior);
}



io_return_t 
cngetstat (dev, flavor, data, count)
	unsigned	dev;
	int		flavor;
	int *		data;		/* pointer to OUT array */
	unsigned int	*count;		/* OUT */
{
  return tty_get_status (&uart_tty, flavor, data, count);
}


io_return_t 
cnsetstat (dev, flavor, data, count)
	unsigned	dev;
	int		flavor;
	int *		data;		/* pointer to OUT array */
	unsigned int	count;
{
  return tty_set_status (&uart_tty, flavor, data, count);
}

/* low-level brute force output, we poll until the character is out.. */
void 
cnputc(c)
	register int c;
{
  if (c)
    {
      register unsigned int s = spltty();

      /* inhibit tx-interrupts while we poll manually, or the uart driver
         may get `slightly' confused.. */
      custom_p.intena = INTF_TBE;
      while (! (custom_p.serdatr & (SERDATRF_TBE|SERDATRF_TSRE))) ;
      custom_p.serdat = (c&0xff) | 0x100;
      while (! (custom_p.serdatr & SERDATRF_TBE)) ;
      custom_p.intreq = INTF_TBE;
      custom_p.intena = INTF_SETCLR | INTF_TBE;

      if (c == '\n')
	cnputc('\r');
      splx(s);
    }
}

#if MACH_KDB
int ddb_trap_char = -2;
#endif

/*
 * Get a character from console (brute force, overrides uart nice behavior)
 */
int cngetc()
{
    register int s,c;
    
    s = spltty();
    c = custom_p.serdatr & 0xffff;
    if (! (c & SERDATRF_RBF))
      c = -1;
    else
      custom_p.intreq = INTF_RBF;

#if MACH_KDB
    if ( c == ddb_trap_char)
      {
	c = -1;
	gimmeabreak();
      }
#endif
    splx(s);
    return c;
}


int 
cnportdeath(dev, port)
	unsigned	dev;
	mach_port_t	port;
{
	return (tty_portdeath(&uart_tty, (ipc_port_t)port));
}


int 
cnmmap(dev, off, prot)
	unsigned dev;
	off_t off;
	int prot;
{
  return -1;
}

